"""
#!/usr/bin/env -S uv run
# /// script
# requires-python = ">=3.13"
# dependencies = [
#   "rich",
#   "polars",
#   "typer>=0.12",
#   "loguru",
#   "numpy",
# ]
# ///

"""


def main():
    """a,b,c"""
    print("This is a helper function in a.py")
    import numpy as np
    print("Numpy version:", np.__version__)


if __name__ == "__main__":
    main()



"""devops with emojis"""

import typer
from typing import Optional, Annotated

import machineconfig.scripts.python.helpers_devops.cli_repos as cli_repos
import machineconfig.scripts.python.helpers_devops.cli_config as cli_config
import machineconfig.scripts.python.helpers_devops.cli_self as cli_self
import machineconfig.scripts.python.helpers_devops.cli_data as cli_data
import machineconfig.scripts.python.helpers_devops.cli_nw as cli_network
import machineconfig.scripts.python.helpers.run_script as run_py_script_module

def install(which: Annotated[Optional[str], typer.Argument(..., help="Comma-separated list of program names to install, or group name if --group flag is set.")] = None,
        group: Annotated[bool, typer.Option(..., "--group", "-g", help="Treat 'which' as a group name. A group is bundle of apps.")] = False,
        interactive: Annotated[bool, typer.Option(..., "--interactive", "-i", help="Interactive selection of programs to install.")] = False,
    ) -> None:
        """📦 Install packages"""
        import machineconfig.utils.installer_utils.installer_cli as installer_entry_point
        installer_entry_point.main_installer_cli(which=which, group=group, interactive=interactive)



# def get_app():
#     app = typer.Typer(add_completion=False, no_args_is_help=True)
#     app.command(name="scripts", help="define all scripts", no_args_is_help=False)(define_scripts)
#     return app

# def main():
#     # return app
#     app = get_app()
#     app()

#     define_app = get_define_app()


def get_app():

    cli_app = typer.Typer(help="🛠️ DevOps operations", no_args_is_help=True, add_help_option=False, add_completion=False)
    cli_app.command("install", no_args_is_help=True, help="🛠️ [i] Install essential packages")(install)
    cli_app.command("i", no_args_is_help=True, help="Install essential packages", hidden=True)(install)

    app_repos = cli_repos.get_app()
    cli_app.add_typer(app_repos, name="repos")
    cli_app.add_typer(app_repos, name="r", hidden=True)
    app_config = cli_config.get_app()
    cli_app.add_typer(app_config, name="config")
    cli_app.add_typer(app_config, name="c", hidden=True)
    app_data = cli_data.get_app()
    cli_app.add_typer(app_data, name="data")
    cli_app.add_typer(app_data, name="d", hidden=True)
    app_self = cli_self.get_app()
    cli_app.add_typer(app_self, name="self")
    cli_app.add_typer(app_self, name="s", hidden=True)
    app_nw = cli_network.get_app()
    cli_app.add_typer(app_nw, name="network")
    cli_app.add_typer(app_nw, name="n", hidden=True)

    cli_app.command("python", no_args_is_help=True, help="🐍 [p] python/shell scripts or command/file in the machineconfig environment", context_settings={"show_help_on_error": True})(run_py_script_module.run_py_script)
    cli_app.command("p", no_args_is_help=True, help="RUN python scripts or command/file in the machineconfig environment", hidden=True)(run_py_script_module.run_py_script)

    return cli_app


def main():
    app = get_app()
    app()



"""
fire

# https://github.com/pallets/click combine with fire. Consider
# https://github.com/ceccopierangiolieugenio/pyTermTk for choose_from_options build TUI
# https://github.com/chriskiehl/Gooey build commandline interface

"""

from typing import Optional, Annotated
import typer


def route(args: "FireJobArgs", fire_args: str = "") -> None:
    from pathlib import Path
    from machineconfig.utils.path_helper import get_choice_file
    from machineconfig.utils.accessories import get_repo_root, randstr
    choice_file = get_choice_file(args.path, suffixes=None)
    repo_root = get_repo_root(choice_file)
    print(f"💾 Selected file: {choice_file}.\nRepo root: {repo_root}")
    if args.marimo:
        print(f"🧽 Preparing to launch Marimo notebook for `{choice_file}`...")
        tmp_dir = Path.home().joinpath(f"tmp_results/tmp_scripts/marimo/{choice_file.stem}_{randstr()}")
        tmp_dir.mkdir(parents=True, exist_ok=True)
        script = f"""
cd {tmp_dir}
uv run --python 3.14 --with marimo marimo convert {choice_file} -o marimo_nb.py
uv run --project {repo_root} --with marimo marimo edit --host 0.0.0.0 marimo_nb.py
"""
        from machineconfig.utils.code import exit_then_run_shell_script
        print(f"🚀 Launching Marimo notebook for `{choice_file}`...")
        exit_then_run_shell_script(script)
        return

    # =========================  preparing kwargs_dict
    if choice_file.suffix == ".py":
        from machineconfig.scripts.python.helpers_fire_command.fire_jobs_args_helper import extract_kwargs
        kwargs_dict = extract_kwargs(args)  # This now returns empty dict, but kept for compatibility
    else:
        kwargs_dict = {}

    # =========================  choosing function to run
    choice_function: Optional[str] = None  # Initialize to avoid unbound variable
    if args.choose_function:
        from machineconfig.scripts.python.helpers_fire_command.fire_jobs_route_helper import choose_function_or_lines

        choice_function, choice_file, kwargs_dict = choose_function_or_lines(choice_file, kwargs_dict)
    else:
        choice_function = args.function

    if choice_file.suffix == ".py":
        module_line = "-m" if args.module else ""
        with_project = f"--project {repo_root} " if repo_root is not None else ""
        interactive_line = "-i" if args.interactive else ""
        if args.interactive:
            from machineconfig.utils.ve import get_ve_path_and_ipython_profile
            _ve_root_from_file, ipy_profile = get_ve_path_and_ipython_profile(init_path=choice_file)
            if ipy_profile is None:
                ipy_profile = "default"
            ipython_line = f"--no-banner --profile {ipy_profile} "
        else:
            ipython_line = ""

        if args.streamlit:
            from machineconfig.scripts.python.helpers_fire_command.fire_jobs_route_helper import get_command_streamlit
            interpreter_line = get_command_streamlit(choice_file=choice_file, environment=args.environment, repo_root=repo_root)
        elif args.jupyter:
            interpreter_line = "jupyter-lab"
        else:
            interpreter_line = "python" if not args.interactive else "ipython"

        exe_line = f"uv run {with_project} {interpreter_line} {interactive_line} {module_line} {ipython_line}"


    elif choice_file.suffix == ".ps1" or choice_file.suffix == ".sh":
        exe_line = "."
    elif choice_file.suffix == "":
        exe_line = ""
    else:
        raise NotImplementedError(f"File type {choice_file.suffix} not supported, in the sense that I don't know how to fire it.")

    if args.module and choice_file.suffix == ".py":
        if repo_root is not None:
            choice_file_adjusted = ".".join(Path(choice_file).relative_to(repo_root).parts).replace(".py", "")
        else:
            choice_file_adjusted = ".".join(Path(choice_file).relative_to(Path.cwd()).parts).replace(".py", "")
    else:
        choice_file_adjusted = str(choice_file)

    if args.script or (args.debug and args.choose_function):
        # because debugging tools do not support choosing functions and don't interplay with fire module. So the only way to have debugging and choose function options is to import the file as a module into a new script and run the function of interest there and debug the new script.
        assert choice_file == ".py", f"File must be a python file to be imported as a module. Got {choice_file}"
        from machineconfig.scripts.python.helpers_fire_command.file_wrangler import get_import_module_code, wrap_import_in_try_except
        from machineconfig.utils.meta import lambda_to_python_script
        from machineconfig.utils.code import print_code

        import_code = get_import_module_code(str(choice_file))
        import_code_robust = lambda_to_python_script(
            lambda: wrap_import_in_try_except(
                import_line=import_code, pyfile=str(choice_file), repo_root=str(repo_root) if repo_root is not None else None
            ),
            in_global=True,
            import_module=False,
        )
        # print(f"🧩 Preparing import code for module import:\n{import_code}")
        code_printing = lambda_to_python_script(
            lambda: print_code(code=import_code_robust, lexer="python", desc="import as module code"),
            in_global=True, import_module=False
        )
        print(f"🧩 Preparing import code for module import:\n{import_code}")
        if choice_function is not None:
            calling = f"""res = {choice_function}({("**" + str(kwargs_dict)) if kwargs_dict else ""})"""
        else:
            calling = """# No function selected to call. You can add your code here."""
        choice_file = Path.home().joinpath(f"tmp_results/tmp_scripts/python/{Path(choice_file).parent.name}_{Path(choice_file).stem}_{randstr()}.py")
        choice_file.parent.mkdir(parents=True, exist_ok=True)
        choice_file.write_text(import_code_robust + "\n" + code_printing + "\n" + calling, encoding="utf-8")

    if args.debug:
        import platform
        if platform.system() == "Windows":
            command = f"{exe_line} -m ipdb {choice_file_adjusted} "  # pudb is not available on windows machines, use poor man's debugger instead.
        elif platform.system() in ["Linux", "Darwin"]:
            command = f"{exe_line} -m pudb {choice_file_adjusted} "  # TODO: functions not supported yet in debug mode.
        else:
            raise NotImplementedError(f"Platform {platform.system()} not supported.")
    elif args.module:
        # both selected function and kwargs are mentioned in the made up script, therefore no need for fire module.
        command = f"{exe_line} {choice_file_adjusted} "
    elif choice_function is not None and choice_file.suffix == ".py":
        command = f"{exe_line} -m fire {choice_file_adjusted} {choice_function} {fire_args}"
    elif args.streamlit:
        # for .streamlit config to work, it needs to be in the current directory.
        if args.holdDirectory:
            command = f"{exe_line} {choice_file}"
        else:
            command = f"cd {choice_file.parent}\n{exe_line} {choice_file.name}\ncd {Path.cwd()}"
    elif args.cmd:
        command = rf""" cd /d {choice_file.parent} & {exe_line} {choice_file.name} """
    else:
        if choice_file.suffix == "":
            command = f"{exe_line} {choice_file} {fire_args}"
        else:
            command = f"{exe_line} {choice_file} "

    if not args.cmd:
        pass
    else:
        new_line = "\n"
        command = rf"""start cmd -Argument "/k {command.replace(new_line, " & ")} " """  # this works from powershell
    if args.submit_to_cloud:
        command = f"""uv run python -m machineconfig.cluster.templates.cli_click --file {choice_file} """
        if choice_function is not None:
            command += f"--function {choice_function} "

    if args.optimized:
        command = command.replace("python ", "python -OO ")

    from rich.panel import Panel
    from rich.console import Console
    from rich.syntax import Syntax

    console = Console()
    if args.zellij_tab is not None:
        comman_path__ = Path.home().joinpath(f"tmp_results/tmp_scripts/zellij_commands/{choice_file.stem}_{randstr()}.sh")
        comman_path__.parent.mkdir(parents=True, exist_ok=True)
        comman_path__.write_text(command, encoding="utf-8")
        console.print(Panel(Syntax(command, lexer="shell"), title=f"🔥 fire command @ {comman_path__}: "), style="bold red")
        import subprocess

        existing_tab_names = subprocess.run(["zellij", "action", "query-tab-names"], capture_output=True, text=True, check=True).stdout.splitlines()
        if args.zellij_tab in existing_tab_names:
            print(f"⚠️ Tab name `{args.zellij_tab}` already exists. Please choose a different name.")
            args.zellij_tab += f"_{randstr(3)}"
        from machineconfig.cluster.sessions_managers.zellij_local import run_command_in_zellij_tab

        command = run_command_in_zellij_tab(command=str(comman_path__), tab_name=args.zellij_tab, cwd=None)
    if args.watch:
        command = "watchexec --restart --exts py,sh,ps1 " + command
    if args.git_pull:
        command = f"\ngit -C {choice_file.parent} pull\n" + command
    if args.PathExport:
        from machineconfig.scripts.python.helpers_fire_command.file_wrangler import add_to_path

        export_line = add_to_path(path_variable="PYTHONPATH", directory=str(repo_root))
        command = export_line + "\n" + command
    if args.loop:
        import platform
        if platform.system() in ["Linux", "Darwin"]:
            command = command + "\nsleep 0.5"
        elif platform.system() == "Windows":
            command = "$ErrorActionPreference = 'SilentlyContinue';\n" + command + "\nStart-Sleep -Seconds 0.5"
        else:
            raise NotImplementedError(f"Platform {platform.system()} not supported.")
    from machineconfig.utils.code import exit_then_run_shell_script
    exit_then_run_shell_script(script=command, strict=False)


def fire(
    ctx: typer.Context,
    path: Annotated[str, typer.Argument(help="Path to the Python file to run")] = ".",
    function: Annotated[Optional[str], typer.Argument(help="Function to run")] = None,
    ve: Annotated[str, typer.Option("--ve", "-v", help="Virtual environment name")] = "",
    cmd: Annotated[bool, typer.Option("--cmd", "-B", help="Create a cmd fire command to launch the job asynchronously")] = False,
    interactive: Annotated[bool, typer.Option("--interactive", "-i", help="Whether to run the job interactively using IPython")] = False,
    debug: Annotated[bool, typer.Option("--debug", "-d", help="Enable debug mode")] = False,
    choose_function: Annotated[bool, typer.Option("--choose-function", "-c", help="Choose function interactively")] = False,
    loop: Annotated[bool, typer.Option("--loop", "-l", help="Infinite recursion (runs again after completion/interruption)")] = False,
    jupyter: Annotated[bool, typer.Option("--jupyter", "-j", help="Open in a jupyter notebook")] = False,
    marimo: Annotated[bool, typer.Option("--marimo", "-M", help="Open in a marimo notebook")] = False,
    module: Annotated[bool, typer.Option("--module", "-m", help="Launch the main file")] = False,
    script: Annotated[bool, typer.Option("--script", "-s", help="Launch as a script without fire")] = False,
    optimized: Annotated[bool, typer.Option("--optimized", "-O", help="Run the optimized version of the function")] = False,
    zellij_tab: Annotated[Optional[str], typer.Option("--zellij-tab", "-z", help="Open in a new zellij tab")] = None,
    submit_to_cloud: Annotated[bool, typer.Option("--submit-to-cloud", "-C", help="Submit to cloud compute")] = False,
    remote: Annotated[bool, typer.Option("--remote", "-r", help="Launch on a remote machine")] = False,
    streamlit: Annotated[bool, typer.Option("--streamlit", "-S", help="Run as streamlit app")] = False,
    environment: Annotated[str, typer.Option("--environment", "-E", help="Choose ip, localhost, hostname or arbitrary url")] = "",
    holdDirectory: Annotated[
        bool, typer.Option("--holdDirectory", "-D", help="Hold current directory and avoid cd'ing to the script directory")
    ] = False,
    PathExport: Annotated[bool, typer.Option("--PathExport", "-P", help="Augment the PYTHONPATH with repo root")] = False,
    git_pull: Annotated[bool, typer.Option("--git-pull", "-g", help="Start by pulling the git repo")] = False,
    watch: Annotated[bool, typer.Option("--watch", "-w", help="Watch the file for changes")] = False,
) -> None:
    """Main function to process fire jobs arguments."""

    # Get Fire arguments from context
    from machineconfig.scripts.python.helpers_fire_command.fire_jobs_args_helper import FireJobArgs, parse_fire_args_from_context
    fire_args = parse_fire_args_from_context(ctx)

    args = FireJobArgs(
        path=path,
        function=function,
        ve=ve,
        cmd=cmd,
        interactive=interactive,
        debug=debug,
        choose_function=choose_function,
        loop=loop,
        jupyter=jupyter,
        marimo=marimo,
        submit_to_cloud=submit_to_cloud,
        remote=remote,
        module=module,
        script=script,
        streamlit=streamlit,
        environment=environment,
        holdDirectory=holdDirectory,
        PathExport=PathExport,
        git_pull=git_pull,
        optimized=optimized,
        zellij_tab=zellij_tab,
        watch=watch,
    )
    try:
        route(args, fire_args)
    except SystemExit:
        # Re-raise SystemExit to preserve exit codes and allow clean exits
        raise
    except Exception as e:
        # For other exceptions, print clean error message and exit
        import sys

        print(f"❌ Error: {e}", file=sys.stderr)
        sys.exit(1)


def get_app():
    from typer import Typer

    app = Typer(add_completion=False)
    app.command(context_settings={"allow_extra_args": True, "allow_interspersed_args": False})(fire)
    return app


def main():
    app = get_app()
    app()


if __name__ == "__main__":
    from machineconfig.scripts.python.helpers_fire_command.fire_jobs_args_helper import FireJobArgs




import typer
from typing import Annotated


def machineconfig_search(
        directory: Annotated[str, typer.Option(..., "--directory", "-d", help="The directory to search")] = ".",
        ast: Annotated[bool, typer.Option(..., "--ast", "-a", help="The abstract syntax tree search/ tree sitter search of symbols")] = False,
        symantic: Annotated[bool, typer.Option(..., "--symantic", "-s", help="The symantic search of symbols")] = False,
        extension: Annotated[str, typer.Option(..., "--extension", "-e", help="File extension to filter by (e.g., .py, .js)")] = "",
        file: Annotated[bool, typer.Option(..., "--file", "-f", help="File search using fzf")] = False,
        no_dotfiles: Annotated[bool, typer.Option(..., "--no-dotfiles", "-D", help="Exclude dotfiles from search")] = False,
        rga: Annotated[bool, typer.Option(..., "--rga", "-A", help="Use ripgrep-all for searching all (non text files) instead of ripgrep")] = False,
        install_dependencies: Annotated[bool, typer.Option(..., "--install-req", "-p", help="Install required dependencies if missing")] = False
        ):
    if install_dependencies:
        from machineconfig.utils.installer_utils.installer_cli import install_if_missing
        install_if_missing("fzf")
        install_if_missing("tv")
        install_if_missing("bat")
        install_if_missing("fd")
        install_if_missing("rg")  # ripgrep
        install_if_missing("rga")  # ripgrep-all
        # install_if_missing("tree-sitter-cli")
        return
    if symantic:
        script = ""
        for an_ex in extension.split(","):
            script = script + f"""\nparse *.{an_ex} """
        from machineconfig.utils.code import run_shell_script
        run_shell_script(script=script)
        return
    if ast:
        from machineconfig.scripts.python.helpers.ast_search import get_repo_symbols
        symbols = get_repo_symbols(directory)
        from machineconfig.utils.options import choose_from_options
        try:
            res = choose_from_options(options=symbols, msg="Select a symbol to search for:", tv=True, multi=False)
            from rich import print_json
            import json
            res_json = json.dumps(res, indent=4)
            print_json(res_json)
            return None
        except Exception as e:
            print(f"❌ Error during selection: {e}")
            return None
    if file:
        script = """fzf --ansi --preview-window 'right:60%' --preview 'bat --color=always --style=numbers,grid,header --line-range :300 {}' """
        if no_dotfiles:
            script = "fd | " + script
        from machineconfig.utils.code import run_shell_script
        run_shell_script(script=script)
        return
    from machineconfig.scripts.python.helpers_msearch import FZFG_LINUX_PATH, FZFG_WINDOWS_PATH
    import platform
    if platform.system() == "Linux" or platform.system() == "Darwin":
        script = FZFG_LINUX_PATH.read_text(encoding="utf-8")
    elif platform.system() == "Windows":
        script = FZFG_WINDOWS_PATH.read_text(encoding="utf-8")
    else:
        raise RuntimeError("Unsupported platform")
    if rga:
        script = script.replace("rg ", "rga ").replace("ripgrep", "ripgrep-all")
    from machineconfig.utils.code import exit_then_run_shell_script
    exit_then_run_shell_script(script=script, strict=False)


def main():
    app = typer.Typer(add_completion=False, no_args_is_help=True)
    app.command(name="msearch", help="machineconfig search helper", no_args_is_help=False)(machineconfig_search)
    app()




from typing import Literal
from pathlib import Path
import shutil
from machineconfig.utils.source_of_truth import LIBRARY_ROOT, CONFIG_ROOT


def _copy_path(source: Path, target: Path, overwrite: bool = False) -> None:
    source = source.expanduser().resolve()
    target = target.expanduser().resolve()
    if not source.exists():
        raise FileNotFoundError(f"Source path does not exist: {source}")
    target.parent.mkdir(parents=True, exist_ok=True)
    if target.exists() and not overwrite:
        raise FileExistsError(f"Target already exists and overwrite=False: {target}")
    if target.exists() and overwrite:
        if target.is_dir():
            shutil.rmtree(target)
        else:
            target.unlink()
    if source.is_file():
        shutil.copy2(source, target)
    elif source.is_dir():
        shutil.copytree(source, target, dirs_exist_ok=overwrite)
    else:
        raise ValueError(f"Source is neither file nor directory: {source}")


def copy_assets_to_machine(which: Literal["scripts", "settings"]) -> None:
    import platform
    import subprocess
    
    system_name = platform.system().lower()
    if system_name == "windows":
        system = "windows"
    elif system_name in {"linux", "darwin"}:
        system = "linux"
    else:
        raise NotImplementedError(f"System {system_name} not supported")

    match which:
        case "scripts":
            source = LIBRARY_ROOT.joinpath("scripts", system)
            target = CONFIG_ROOT.joinpath("scripts")
            
            wrap_mcfg_source = LIBRARY_ROOT.joinpath("scripts", "nu", "wrap_mcfg.nu")
            wrap_mcfg_target = CONFIG_ROOT.joinpath("scripts", "wrap_mcfg.nu")
            wrap_mcfg_target.parent.mkdir(parents=True, exist_ok=True)
            _copy_path(source=wrap_mcfg_source, target=wrap_mcfg_target, overwrite=True)
        case "settings":
            source = LIBRARY_ROOT.joinpath("settings")
            target = CONFIG_ROOT.joinpath("settings")

    _copy_path(source=source, target=target, overwrite=True)
    
    if system_name == "linux" and which == "scripts":
        from rich.console import Console
        console = Console()
        console.print("\n[bold]📜 Setting executable permissions for scripts...[/bold]")
        scripts_path = CONFIG_ROOT.joinpath("scripts")
        subprocess.run(f"chmod +x {scripts_path} -R", shell=True, capture_output=True, text=True, check=False)
        console.print("[green]✅ Script permissions updated[/green]")





import typer
from typing import Annotated, Optional, Literal


def run_py_script(name: Annotated[str, typer.Argument(help="Name of the python script to run, e.g., 'a' for a.py")],
                  where: Annotated[Literal["all", "private", "public", "tmp"], typer.Option("--where", "-w", help="Where to look for the script: any, private, public, tmp")] = "all",
                  command: Annotated[Optional[bool], typer.Option(..., "--command", "-c", help="Run as command")] = False,
                #   use_machineconfig_env: Annotated[bool, typer.Option(..., "--use-machineconfig-env/--no-use-machineconfig-env", "-m/-nm", help="Whether to use the machineconfig python environment")] = False
                ) -> None:
    """
    Run a temporary python script stored in machineconfig/scripts/python/helpers/tmp_py_scripts.
    """
    from machineconfig.utils.source_of_truth import CONFIG_ROOT
    from pathlib import Path
    if command:
        exec(name)
        return
    if Path(name).is_file():
        if name.endswith(".py"):
            import machineconfig
            import subprocess
            import sys
            subprocess.run([sys.executable, name], cwd=machineconfig.__path__[0])
            return
        else:
            raise RuntimeError(f"File '{name}' is not a python (.py) file.")

    private_python_root = Path.home().joinpath("dotfiles/scripts/python")
    public_python_root = CONFIG_ROOT.joinpath("scripts_python")

    from machineconfig.utils.source_of_truth import LIBRARY_ROOT
    repo_python_root = LIBRARY_ROOT.joinpath("jobs", "scripts", "python_scripts")

    import platform
    if platform.system() == "Windows":
        private_shell_root = Path.home().joinpath("dotfiles/scripts/windows")
        public_shell_root = CONFIG_ROOT.joinpath("scripts_powershell")
        repo_shell_root = LIBRARY_ROOT.joinpath("jobs", "scripts", "powershell_scripts")
    elif platform.system() == "Darwin" or platform.system() == "Linux":
        private_shell_root = Path.home().joinpath("dotfiles/scripts/linux")
        public_shell_root = CONFIG_ROOT.joinpath("scripts_bash")
        repo_shell_root = LIBRARY_ROOT.joinpath("jobs", "scripts", "bash_scripts")
    else:
        raise NotImplementedError(f"Platform {platform.system()} not supported for shell scripts.")

    _ = private_shell_root, public_shell_root, repo_shell_root

    target_file: Optional[Path] = None
    candidates: list[Path] = []
    if where in ["all", "private"]:
        result = private_python_root.joinpath(f"{name}.py")
        if result.exists():
            target_file = result
        else:
            candidates.extend(private_python_root.rglob("*.py"))
    if target_file is None and where in ["all", "public"]:
        result = public_python_root.joinpath(f"{name}.py")
        if result.exists():
            target_file = result
        else:
            candidates.extend(public_python_root.rglob("*.py"))
    if target_file is None and where in ["all", "tmp"]:
        try:
            # src/machineconfig/jobs/scripts/python_scripts/a.py
            url = f"""https://raw.githubusercontent.com/thisismygitrepo/machineconfig/refs/heads/main/src/machineconfig/jobs/scripts/python_scripts/{name}.py"""
            print(f"Fetching temporary script from {url} ...")
            import requests
            response = requests.get(url)
            if response.status_code != 200:
                print(f"❌ ERROR: Could not fetch script '{name}.py' from repository. Status Code: {response.status_code}")
                raise RuntimeError(f"Could not fetch script '{name}.py' from repository.")
            script_content = response.text
            target_file = Path.home().joinpath("tmp_results", "tmp_scripts", "python", f"{name}.py")
            target_file.parent.mkdir(parents=True, exist_ok=True)
            target_file.write_text(script_content, encoding="utf-8")
        except Exception as e:
            print(f"❌ ERROR: Could not fetch script '{name}.py' from repository due to error: {e}")
            candidates += repo_python_root.rglob("*.py")

    if target_file is None:
        print(f"❌ ERROR: Could not find script '{name}.py'. Checked {len(candidates)} candidate files, trying interactively:")
        from machineconfig.utils.options import choose_from_options
        chosen_file = choose_from_options([str(p) for p in candidates], multi=False, msg="Select the script to run:")
        target_file = Path(chosen_file)

    from machineconfig.utils.code import get_uv_command_executing_python_script, exit_then_run_shell_script
    script_content = target_file.read_text(encoding="utf-8")
    shell_script, _shell_script_path = get_uv_command_executing_python_script(python_script=script_content, uv_project_dir=None, uv_with=None, prepend_print=False)
    exit_then_run_shell_script(script=shell_script)



def copy_script_to_local(name: Annotated[str, typer.Argument(help="Name of the temporary python script to copy, e.g., 'a' for a.py")],
                         alias: Annotated[Optional[str], typer.Option("--alias", "-a", help="Whether to create call it a different name locally")] = None
                         ) -> None:
    """
    Copy a temporary python script stored in machineconfig/scripts/python/helpers/tmp_py_scripts to the local machine.
    """
    url = f"""https://raw.githubusercontent.com/thisismygitrepo/machineconfig/refs/heads/main/src/machineconfig/scripts/python/helpers/tmp_py_scripts/{name}.py"""
    import requests
    response = requests.get(url)
    if response.status_code != 200:
        print(f"❌ ERROR: Could not fetch script '{name}.py' from repository. Status Code: {response.status_code}")
        raise RuntimeError(f"Could not fetch script '{name}.py' from repository.")
    script_content = response.text
    from machineconfig.utils.source_of_truth import CONFIG_ROOT
    local_path = CONFIG_ROOT.joinpath(f"scripts_python/{alias or name}.py")
    with open(local_path, "w") as f:
        f.write(script_content)
    print(f"✅ Script '{name}.py' has been copied to '{local_path}'.")


def get_app():
    app = typer.Typer(
        name="run-tmp-script",
        help="Helper to run temporary python scripts stored in machineconfig/scripts/python/helpers/tmp_py_scripts",
        no_args_is_help=True,
    )
    from machineconfig.jobs.scripts.python_scripts import a
    app.command()(a.main)
    return app




from pathlib import Path

UV = Path(__file__).parent.joinpath("uv.ps1")
DOCKER = Path(__file__).parent.joinpath("others/docker.ps1")
OBS = Path(__file__).parent.joinpath("others/obs.ps1")
POWER_OPTIONS = Path(__file__).parent.joinpath("others/power_options.ps1")
SSH_SERVER = Path(__file__).parent.joinpath("ssh/openssh_server.ps1")
INTERACTIVE = Path(__file__).parent.joinpath("web_shortcuts/interactive.ps1")
LIVE = Path(__file__).parent.joinpath("web_shortcuts/live_from_github.ps1")




#!/usr/bin/env -S uv run --no-dev --project

"""
croshell

"""

from typing import Annotated, Optional
import typer


def croshell(
    path: Annotated[Optional[str], typer.Argument(help="path of file to read.")] = None,
    project_path: Annotated[Optional[str], typer.Option("--project", "-p", help="specify uv project to use")] = None,
    uv_with: Annotated[Optional[str], typer.Option("--uv-with", "-w", help="specify uv with packages to use")] = None,
    marimo: Annotated[bool, typer.Option("--marimo", "-m", help="open the notebook using marimo if available")] = False,
    jupyter: Annotated[bool, typer.Option("--jupyter", "-j", help="run in jupyter interactive console")] = False,
    vscode: Annotated[bool, typer.Option("--vscode", "-c", help="open the script in vscode")] = False,
    visidata: Annotated[bool, typer.Option("--visidata", "-v", help="open data file in visidata")] = False,
    # streamlit_viewer: Annotated[bool, typer.Option("--streamlit", "-s", help="view in streamlit app")] = False,
    python: Annotated[bool, typer.Option("--python", "-P", help="flag to use python over IPython.")] = False,
    profile: Annotated[Optional[str], typer.Option("--profile", "-r", help="ipython profile to use, defaults to default profile.")] = None,
    
) -> None:
    if uv_with is not None: user_uv_with_line = f"--with {uv_with} "
    else: user_uv_with_line = ""

    if project_path is not None:
        uv_project_line = f'--project {project_path}'
        uv_python_line = ""
    else:
        uv_project_line = ""
        uv_python_line = "--python 3.14"

    from machineconfig.scripts.python.helpers_croshell.crosh import get_read_python_file_pycode, get_read_data_pycode
    from machineconfig.utils.meta import lambda_to_python_script
    from pathlib import Path
    from machineconfig.utils.accessories import randstr
    from machineconfig.utils.ve import get_ve_path_and_ipython_profile
    import json
    from rich.console import Console
    from rich.panel import Panel
    console = Console()


    # ==================================================================================
    # flags processing
    interactivity = "-i"
    interpreter = "python" if python else "ipython"
    ipython_profile: Optional[str] = profile
    file_obj = Path.cwd()  # initialization value, could be modified according to args.
    if path is not None:
        from machineconfig.utils.path_helper import get_choice_file
        choice_file = get_choice_file(path=path, suffixes={".*"})
        if project_path is None:
            ve_path, _ = get_ve_path_and_ipython_profile(choice_file)
            if ve_path is not None:
                ve_path_obj = Path(ve_path)
                uv_project_line = f'--project {ve_path_obj.parent}'
                uv_python_line = ""
        if choice_file.suffix == ".py":
            program = choice_file.read_text(encoding="utf-8")
            text = f"📄 Selected file: {choice_file.name}"
            console.print(Panel(text, title="[bold blue]Info[/bold blue]"))
        else:
            program = lambda_to_python_script(lambda: get_read_data_pycode(path=str(choice_file)),
                                              in_global=True, import_module=False)
            text = f"📄 Reading data from: {file_obj.name}"
            console.print(Panel(text, title="[bold blue]Info[/bold blue]"))
    else:  # if nothing is specified, then run in interactive mode.
        program = ""

    if Path.home().joinpath("code/machineconfig").exists() and uv_project_line == "":
        uv_project_line = f'--project "{str(Path.home().joinpath("code/machineconfig"))}"'

    preprogram = """
#%%
"""
    def preprogram_func():
        from machineconfig.utils.files.headers import print_header, print_logo
        print_header()
        print_logo("Machineconfig")
        from pathlib import Path
        from machineconfig.utils.path_extended import PathExtended
        _ = Path, PathExtended  # avoid unused import warnings
    import inspect
    import textwrap
    from types import FunctionType
    def get_body_simple_function_no_args(f: FunctionType):
        return textwrap.dedent("\n".join(inspect.getsource(f).splitlines()[1:]))
    preprogram += get_body_simple_function_no_args(preprogram_func)

    from pathlib import Path
    pyfile = Path.home().joinpath(f"tmp_results/tmp_scripts/python/croshell/{randstr()}/script.py")
    pyfile.parent.mkdir(parents=True, exist_ok=True)
    title = "Reading Data"
    def_code = lambda_to_python_script(lambda: get_read_python_file_pycode(path=str(pyfile), title=title),
                                       in_global=False, import_module=False)
    # print(def_code)
    python_program = preprogram + "\n\n" + def_code + program
    pyfile.write_text(python_program, encoding="utf-8")
    # ve_root_from_file, ipython_profile = get_ve_path_and_ipython_profile(PathExtended(file))
    ipython_profile = ipython_profile if ipython_profile is not None else "default"
    # ve_activateion_line = get_ve_activate_line(ve_name=args.ve or ve_profile_suggested, a_path=str(PathExtended.cwd()))

    # prepare notebook target path (avoid relying on locals())
    nb_target = pyfile.with_suffix(".ipynb")
    if jupyter:
        try:
            nb_path = pyfile.with_suffix(".ipynb")
            nb_content = {
                "cells": [
                    {
                        "cell_type": "code",
                        "metadata": {"language": "python"},
                        "source": [python_program],
                        "outputs": [],
                        "execution_count": None,
                    }
                ],
                "metadata": {},
                "nbformat": 4,
                "nbformat_minor": 5,
            }
            nb_path.write_text(json.dumps(nb_content), encoding="utf-8")
            nb_target = nb_path
        except Exception:
            # if writing fails, fall back to the default nb_target already set
            pass
    if visidata:
        if file_obj.suffix == ".json":
            fire_line = f"uv run {uv_python_line} {user_uv_with_line} {uv_project_line} --with visidata vd {str(file_obj)}"
        else:
            fire_line = f"uv run {uv_python_line} {user_uv_with_line} {uv_project_line} --with visidata,pyarrow vd {str(file_obj)}"
    elif marimo:
        if Path.home().joinpath("code/machineconfig").exists():
            requirements = f"""{user_uv_with_line} {uv_project_line} --with marimo  """
        else: requirements = f"""{uv_python_line} {user_uv_with_line} {uv_project_line} --with "marimo,cowsay,machineconfig[plot]>=7.98" """
        fire_line = f"""
cd {str(pyfile.parent)}
uv run {uv_python_line} --with "marimo" marimo convert {pyfile.name} -o marimo_nb.py
uv run {requirements} marimo edit --host 0.0.0.0 marimo_nb.py
"""
    elif jupyter:
        if Path.home().joinpath("code/machineconfig").exists():
            requirements = f"""{user_uv_with_line}  {uv_project_line}  --with jupyterlab """
        else: requirements = f"""{user_uv_with_line} {uv_project_line} --with "cowsay,machineconfig[plot]>=7.98" """
        fire_line = f"uv run {requirements} {uv_project_line}  jupyter-lab {str(nb_target)}"
    elif vscode:
        user_uv_add = f"uv add {uv_with}" if uv_with is not None else ""
        fire_line = f"""
cd {str(pyfile.parent)}
uv init {uv_python_line}
uv venv
uv add "cowsay,machineconfig[plot]>=7.98"
uv add {user_uv_add}
# code serve-web
code --new-window {str(pyfile)}
"""
    else:
        if interpreter == "ipython": profile = f" --profile {ipython_profile} --no-banner"
        else: profile = ""
        if Path.home().joinpath("code/machineconfig").exists():
            ve_line = f"""{user_uv_with_line}  {uv_project_line} """
        else: ve_line = f"""{uv_python_line} {user_uv_with_line} {uv_project_line} --with "cowsay,machineconfig[plot]>=7.98" """
        fire_line = f"uv run {ve_line} {interpreter} {interactivity} {profile} {str(pyfile)}"

    from machineconfig.utils.code import exit_then_run_shell_script
    exit_then_run_shell_script(fire_line, strict=False)


def main() -> None:
    typer.run(croshell)


if __name__ == "__main__":
    # def func(flag: Annotated[bool, typer.Option("--flag/-nf", help="dummy flag for debugging", flag_value=False, is_flag=True)]=True):
    #     console.print(f"flag: {flag}")
    # app = typer.Typer()
    # app.command()(func)
    # app()
    main()



import typer
from typing import Optional, Annotated, Literal, TypedDict, cast


def tui_env(which: Annotated[Literal["PATH", "p", "ENV", "e"], typer.Argument(help="Which environment variable to display.")] = "ENV") -> None:
    """📚 NAVIGATE PATH variable with TUI"""
    from machineconfig.scripts.python import env_manager as navigator
    from pathlib import Path

    match which:
        case "PATH" | "p":
            path = Path(navigator.__file__).resolve().parent.joinpath("path_manager_tui.py")
        case "ENV" | "e":
            path = Path(navigator.__file__).resolve().parent.joinpath("env_manager_tui.py")
    from machineconfig.utils.code import run_shell_script, get_uv_command_executing_python_script

    uv_with = ["textual"]
    uv_project_dir = None
    if not Path.home().joinpath("code/machineconfig").exists():
        uv_with.append("machineconfig>=7.98")
    else:
        uv_project_dir = str(Path.home().joinpath("code/machineconfig"))
    run_shell_script(
        get_uv_command_executing_python_script(python_script=path.read_text(encoding="utf-8"), uv_with=uv_with, uv_project_dir=uv_project_dir)[0]
    )


def init_project(
    name: Annotated[Optional[str], typer.Option("--name", "-n", help="Name of the project.")] = None,
    tmp_dir: Annotated[
        bool, typer.Option("--tmp-dir", "-t", help="Use a temporary directory for the project initialization.")
    ] = False,
    python: Annotated[Literal["3.13", "3.14"], typer.Option("--python", "-p", help="Python version for the uv virtual environment.")] = "3.13",
    libraries: Annotated[Optional[str], typer.Option("--libraries", "-l", help="Additional packages to include in the uv virtual environment.")] = None,
    group: Annotated[Optional[str], typer.Option("--group", "-g", help="group of packages names (no separation) p:plot, t:types, l:linting, i:interactive, d:data")] = "ptlid",
    # types_packages: Annotated[
    #     bool, typer.Option("--types-packages/--no-types-packages", "-T/-NT", help="Include types packages for better type hinting.")
    # ] = True,
    # linting_debug_packages: Annotated[
    #     bool, typer.Option("--linting-debug-packages/--no-linting-debug-packages", "-L/-NL", help="Include linting and debugging packages.")
    # ] = True,
    # ia_packages: Annotated[bool, typer.Option("--ia-packages/--no-ia-packages", "-I/-NI", help="Include interactive and IA packages.")] = True,
    # plot_packages: Annotated[bool, typer.Option("--plot-packages/--no-plot-packages", "-P/-NP", help="Include plotting packages.")] = True,
    # data_packages: Annotated[bool, typer.Option("--data-packages/--no-data-packages", "-D/-ND", help="Include data manipulation packages.")] = True,

) -> None:
    if libraries is not None:
        packages_add_line = f"uv add {libraries}"
    else:
        packages_add_line = ""
    from pathlib import Path
    if not tmp_dir:
        repo_root = Path.cwd()
        if not (repo_root / "pyproject.toml").exists():
            typer.echo(f"❌ Error: pyproject.toml not found in {repo_root}", err=True)
            raise typer.Exit(code=1)
        starting_code = ""
    else:
        if name is not None:
            from machineconfig.utils.accessories import randstr
            repo_root = Path.home().joinpath(f"tmp_results/tmp_projects/{name}")
        else:
            from machineconfig.utils.accessories import randstr
            repo_root = Path.home().joinpath(f"tmp_results/tmp_projects/{randstr(6)}")
        repo_root.mkdir(parents=True, exist_ok=True)
        print(f"Using temporary directory for project initialization: {repo_root}")
        starting_code = f"""
cd {repo_root}
uv init --python {python}
uv venv
"""
    print(f"Adding group `{group}` with common data science and plotting packages...")
    total_packages: list[str] = []
    if group is not None:
        if "t" in group:
            total_packages.append(
                "types-python-dateutil types-pyyaml types-requests types-tqdm types-mysqlclient types-paramiko types-pytz types-sqlalchemy types-toml types-urllib3"
            )
        if "l" in group:
            total_packages.append("mypy pyright ruff pylint pyrefly cleanpy ipdb pudb")
        if "i" in group:
            total_packages.append("ipython ipykernel jupyterlab nbformat marimo")
        if "p" in group:
            total_packages.append("python-magic matplotlib plotly kaleido")
        if "d" in group:
            total_packages.append("numpy pandas polars duckdb-engine sqlalchemy  psycopg2-binary pyarrow tqdm openpyxl")
    from machineconfig.utils.ve import get_ve_activate_line

    script = f"""
{starting_code}
{packages_add_line}
uv add --group {group} {" ".join(total_packages)}
{get_ve_activate_line(ve_root=str(repo_root.joinpath(".venv")))}
ls
"""
    from machineconfig.utils.code import exit_then_run_shell_script

    exit_then_run_shell_script(script)


def edit_file_with_hx(
    path: Annotated[Optional[str], typer.Argument(..., help="The root directory of the project to edit, or a file path.")] = None,
) -> None:
    from pathlib import Path

    if path is None:
        root_path = Path.cwd()
        print(f"No path provided. Using current working directory: {root_path}")
    else:
        root_path = Path(path).expanduser().resolve()
        print(f"Using provided path: {root_path}")
    from machineconfig.utils.accessories import get_repo_root

    repo_root = get_repo_root(root_path)
    if repo_root is not None and repo_root.joinpath("pyproject.toml").exists():
        code = f"""
cd {repo_root}
uv add --dev pylsp-mypy python-lsp-server[all] pyright ruff-lsp  # for helix editor.
source ./.venv/bin/activate
"""
    else:
        code = ""
    if root_path.is_file():
        code += f"hx {root_path}"
    else:
        code += "hx"
    from machineconfig.utils.code import exit_then_run_shell_script

    exit_then_run_shell_script(code)


class MachineSpecs(TypedDict):
    system: Literal["Windows", "Linux", "Darwin"]
    distro: str
    home_dir: str
    hostname: str
    release: str
    version: str
    machine: str
    processor: str
    python_version: str
    user: str


def get_machine_specs() -> MachineSpecs:
    """Write print and return the local machine specs."""
    import platform
    from machineconfig.utils.code import get_uv_run_command

    uv_run_cmd = get_uv_run_command(platform=platform.system())  # type: ignore
    command = f"""{uv_run_cmd} --with distro python -c "import distro; print(distro.name(pretty=True))" """
    import subprocess
    from pathlib import Path
    import socket
    import os

    distro = subprocess.run(command, shell=True, capture_output=True, text=True).stdout.strip()
    system = platform.system()
    if system not in {"Windows", "Linux", "Darwin"}:
        system = "Linux"
    specs: MachineSpecs = {
        "system": cast(Literal["Windows", "Linux", "Darwin"], system),
        "distro": distro,
        "home_dir": str(Path.home()),
        "hostname": socket.gethostname(),
        "release": platform.release(),
        "version": platform.version(),
        "machine": platform.machine(),
        "processor": platform.processor() or "Unknown",
        "python_version": platform.python_version(),
        "user": os.getenv("USER") or os.getenv("USERNAME") or "Unknown",
    }
    print(specs)
    from machineconfig.utils.source_of_truth import CONFIG_ROOT

    path = CONFIG_ROOT.joinpath("machine_specs.json")
    CONFIG_ROOT.mkdir(parents=True, exist_ok=True)
    import json

    path.write_text(json.dumps(specs, indent=4), encoding="utf-8")
    return specs


if __name__ == "__main__":
    get_machine_specs()
